;***************************************************************************
;* PROGRAM
;*	ADC using the serial SPI port with the maxin 176.
;*	reads the spi port twice to get 16 bit data. stores in bit_lo and bit_hi
;*	and uses delays for timing	
;*
;*	AUTHOR: Adrian Cottle, Electronic Techinian, The University of Bath
;*	DATE: May 2000
;* 	AVR: STK200 Board with AT90S8515 device
;***************************************************************************
              
.include "d:\avr\include\8515def.inc"   

.def bit_lo	=r1
.def bit_hi	=r2
.def temp	=r16
.def coarse	=r18
.def medium	=r19
.def fine	=r20


;*********************** 8515 Interupt Vectors ***********************************

		rjmp Init
		reti			; External 0 interrupt  Vector 
		reti			; External 1 interrupt  Vector 
		reti			; Timer 1 Capture  Vector 
		reti			; Timer1 CompareA  Vector 
		reti			; Timer 1 CompareB  Vector 
		reti			; Timer 1 Overflow  Vector 
		reti			; Timer 0 Overflow  Vector 
		reti			; SPI  Vector 
		reti			; UART Receive  Vector 
		reti			; UDR Empty  Vector 
		reti			; UART Transmit  Vector 
		reti			; Analogue Comparator  Vector 


;********************* Intialise routines **************************************
 

Init:                
		; **** Stack Pointer Setup Code ****  
		ldi temp,$02		; Stack Pointer Setup 
		out SPH,temp		; Stack Pointer High Byte 
		ldi temp,$5F		; Stack Pointer Setup 
		out SPL,temp		; Stack Pointer Low Byte 
	                   
		; **** Comparator Setup Code ****  
		ldi temp,$80		; 
		out ACSR,temp		; switch off comparator 
                       
		
             	; **** Port B Setup Code ****  
		ldi temp,0b10111111	; I/O Direction Values 
		out DDRB,temp		; Port B Direction Register
		ldi temp,0b11111111	; Init value 
		out PORTB,temp		; Port B value

		; **** Port D Setup Code ****  
		ldi temp,0b11111111	; I/O Direction Values 
		out DDRD,temp		; Port B Direction Register
		ldi temp,0b00000000	; Init value 
		out PORTD,temp		; Port B value



;******************************   MAIN PROGRAM   ************************************


main:	rcall readspi		;rcall readspi routine
	rjmp main		;repeat






; *************************  Read SPI ADC routine  ***************************

readspi:
	ldi temp,0B11011000			;set up spi control register
	out spcr,temp	

	sbi PORTB,0				;set conversion high
	ldi temp,0B10011001			;R/W first byte of control word
	out spdr,temp

	in bit_lo,spdr				;copy spdr register to bit_lo register

	rcall delay

	ldi temp,0b01010101
	out spdr,temp				;R/W second byte of control word
		
	cbi PORTB,0				;set converstion low
	
	in bit_hi,spdr				;copy spdr register to bit_hi register
	
	out PORTD,bit_hi			;output hi-byte to PORTD
	out PORTA,bit_lo			;out lo-byte to PORTA

	rcall delay				;short delay
	
	ret	
	


;******** Delay routine ********

delay:	ldi medium,$ff		;load medium with $ff
magain:		ldi fine,$ff	;load fine with $ff
fagain:		dec fine	;decrement fine count
		brne fagain	;branch if equal to fagain
outf: 	dec medium		;decrement meduim count
	brne magain		;branch if equal to magain
outm:	ret			;return
	